/*
 *  streams library
 *
 *  Copyright (C) 2011-2012 by Christian Bockermann, Hendrik Blom
 * 
 *  streams is a library, API and runtime environment for processing high
 *  volume data streams. It is composed of three submodules "stream-api",
 *  "stream-core" and "stream-runtime".
 *
 *  The streams library (and its submodules) is free software: you can 
 *  redistribute it and/or modify it under the terms of the 
 *  GNU Affero General Public License as published by the Free Software 
 *  Foundation, either version 3 of the License, or (at your option) any 
 *  later version.
 *
 *  The stream.ai library (and its submodules) is distributed in the hope
 *  that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
 *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package stream.io;

import java.util.concurrent.LinkedBlockingQueue;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import stream.Data;

/**
 * <p>
 * The default implementation of a blocking queue of data items.
 * </p>
 * 
 * @author Hendrik Blom, Christian Bockermann
 * 
 */
public class BlockingQueue extends AbstractQueue {

	static Logger log = LoggerFactory.getLogger(BlockingQueue.class);

	protected boolean closed = false;
	protected LinkedBlockingQueue<Data> queue;

	/**
	 * @see stream.io.Source#init()
	 */
	@Override
	public void init() throws Exception {
		if (getLimit() < 1) {
			throw new IllegalArgumentException("Invalid queue-size '"
					+ getLimit() + "'!");
		}

		queue = new LinkedBlockingQueue<Data>(getLimit());
	}

	/**
	 * @see stream.io.Stream#close()
	 */
	public void close() throws Exception {
		queue.clear();
		queue.add(Data.END_OF_STREAM);
		queue.notifyAll();
		closed = true;
	}

	/**
	 * @see stream.io.AbstractStream#readItem(stream.Data)
	 */
	@Override
	public Data read() throws Exception {

		Data item = null;
		try {
			if (closed)
				return null;
			item = queue.take();
			log.debug("took item from queue: {}", item);
		} catch (InterruptedException e) {
			if (closed)
				return null;
			else {
				log.error("Interruped while waiting for data: {}",
						e.getMessage());
				if (log.isDebugEnabled())
					e.printStackTrace();
			}
		}

		if (closed || item == Data.END_OF_STREAM) {
			log.debug("Next data-item is end-of-stream event!");
			closed = true;
			return null;
		}

		return item;
	}

	/**
	 * @see stream.io.QueueService#poll()
	 */
	public Data poll() {
		return queue.poll();
	}

	public Data take() {
		try {
			Data item = queue.take();
			if (item == Data.END_OF_STREAM)
				return null;
			return item;
		} catch (Exception e) {
			log.error("Interrupted while reading on queue: {}", e.getMessage());
			if (log.isDebugEnabled())
				e.printStackTrace();
			return null;
		}
	}

	/**
	 * @see stream.io.QueueService#enqueue(stream.Data)
	 */
	public boolean enqueue(Data item) {
		try {
			synchronized (queue) {
				if (!closed)
					queue.put(item);
				queue.notifyAll();
			}
			return true;
		} catch (Exception e) {
			log.error("Error enqueuing item: {}", e.getMessage());
			if (log.isDebugEnabled())
				e.printStackTrace();
			return false;
		}
	}

	/**
	 * @see stream.io.Sink#write(stream.Data)
	 */
	@Override
	public void write(Data item) throws Exception {
		synchronized (queue) {
			if (closed) {
				queue.notifyAll();
				log.error("Queue already closed.");
				// throw new Exception("Queue already closed.");
			}
			queue.put(item);
			queue.notifyAll();
		}
	}

	/**
	 * @see stream.io.QueueService#level()
	 */
	@Override
	public int level() {
		return queue.size();
	}

	/**
	 * @see stream.io.QueueService#capacity()
	 */
	@Override
	public int capacity() {
		return getLimit();
	}

	/**
	 * @see stream.service.Service#reset()
	 */
	@Override
	public void reset() throws Exception {
		if (queue == null) {
			queue = new LinkedBlockingQueue<Data>(this.getLimit());
		} else {
			queue.clear();
		}
	}
}